//
// ASN.java --
//
// Author: Pekka Nikander <pnr@tequila.nixu.fi>
//
// Copyright 1996 by Nixu Oy, Helsinki, Finland
//
// Created: Wed Mar 20 18:22:04 1996
// Last modified: Thu Apr 11 08:59:38 1996 pnr
//
// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.asn1;

import java.util.Dictionary;
import java.util.Hashtable;

import cryptiX.asn1.ASN1Object;
import cryptiX.asn1.OBJECTFactory;

/**
 * The ASN class represents the ASN.1 types.  The ASN.1 types are
 * used when communicating with any SNMP agent.  Each SNMP attribute
 * has a specific ASN.1 type.  These types are represented at
 * the Java level with ASN.XXX literals.
 * <P>
 * This class also acts as an factory for creating ASN.1 OBJECT subclasses
 * for various kinds of Java objects.
 */

public class ASN {
  /**
   * The current Java syntax is broken, circumvent this.
   */
  private static Class _OBJECT;
  static {
    try {
      _OBJECT = Class.forName("cryptiX.asn1.ASN1Object");
    } catch (ClassNotFoundException e) {
      System.err.println("Java does not support Class literals "
			 + "(cryptiX.asn1.ASN1Object)");
    }
  }

  /**
   * A prime suitable for creating hashtables based on
   * ASN types.
   */
  static Hashtable tag2name = new Hashtable(27);

  /**
   * Publicly known ASN.1 types
   */
  public final static ASN UNKNOWN           = new ASN(0, "UNKNOWN");
  public final static ASN BOOLEAN 	        = new ASN(1, "BOOLEAN");
  public final static ASN INTEGER           = new ASN(2, "INTEGER");
  public final static ASN BIT_STRING        = new ASN(3, "BIT STRING");
  public final static ASN OCTET_STRING      = new ASN(4, "OCTET STRING");
  public final static ASN STRING 	        = OCTET_STRING;
  public final static ASN NULL              = new ASN(5, "NULL");
  public final static ASN OID               = new ASN(6, "OID");
  public final static ASN OBJECT_DESCRIPTOR = new ASN(7, "OBJECT DESCRIPTOR");
  public final static ASN EXTERNAL          = new ASN(8, "EXTERNAL");
  public final static ASN REAL              = new ASN(9, "REAL");
  public final static ASN ENUMERATED        = new ASN(10, "ENUMERATED");
  public final static ASN PRINTABLESTRING   = new ASN(19, "PRINTABLESTRING");
  public final static ASN T61STRING         = new ASN(20, "T61STRING");
  public final static ASN IA5STRING         = new ASN(22, "IA5STRING");
  public final static ASN UTCTIME           = new ASN(23, "UTCTIME");
  public final static ASN SEQUENCE          = new ASN(0x30, "SEQUENCE");
  public final static ASN SET               = new ASN(0x31, "SET");
  public final static ASN CONTEXT_SPECIFIC  = new ASN(0x80, "CONTEXTSPECIFIC");

  /**
   * Special object denoting the NULL asn *value* (not NULL type).
   * This can be used when the generic interface requires that
   * a value is given for a NULL type.
   */
  public final static Object Null = new Object();

  /**
   * A prime suitable for creating hashtables based on
   * ASN types.
   */
  final static int INITIAL_HASH_SIZE = 27;

  /**
   * The ASN UNIVERSAL type tag represented by this object.
   */
  protected int tag;

  /**
   * A printable name of this object
   */
  protected String name;

  /**
   * Create a new ASN.1 type with given tag and name.
   */
  protected ASN(int tag, String name) {
    this.tag  = tag;
    this.name = name;
  }

  /**
   * Return the known Java classes the given ASN.1 type
   * can be represented as.
   */
  public static Class[] getTypes(ASN type) {
    try {
      return create(type).getTypes();
    } catch (InstantiationException e) {
      return new Class[0];
    }
  }

  /****************************************************************
   * Factory methods and ASN.1 type repository
   ****************************************************************/

  /**
   * System wide factory
   */
  static final OBJECTFactory factory = new OBJECTFactory();

  /**
   * Produces a new ASN.1 object, given the ASN.1 type name of it.
   * If given null, returns a ASN.1 NULL object.
   * @exception InstantiationException if the internal factory
   *		couldn't create an instance of requested type
   *
   */
  public static ASN1Object create(ASN type) throws InstantiationException {
    ASN1Object instance;

    if (type == null) type = NULL;
    instance = (ASN1Object)factory.create(_OBJECT, type, false);
    instance.init(type);
    return instance;
  }

  /**
   * Produces a new ASN.1 object, given the ASN.1 type and Java value
   * for it.
   * @exception InstantiationException if the internal factory
   *		couldn't create an instance of requested type
   */
  public static ASN1Object create(ASN type, Object value)
    throws InstantiationException {
    ASN1Object instance;

    if (type == null) type = NULL;
    instance = (ASN1Object)factory.create(_OBJECT, type, false);
    instance.init(type);
    instance.setValue(value);
    return instance;
  }

  /**
   * Registers a new ASN.1 type.  Called by the OBJECT
   * classes' static initializers to register those ASN.1 types that
   * the OBJECT classes know of.
   */
  protected static void register(ASN type, Class cl) {
    factory.register(_OBJECT, type, cl);
    tag2name.put(new Integer(type.tag), type.name);     // Hashtable for tag - name
  }

  /****************************************************************
   * java.lang.Object interface
   ****************************************************************/

  /**
   * Returns a String that represents the value of this Object.
   */
  public String toString() {
    return getClass().getName() + "[" + tag + "," + name + "]";
  }

  /**
   * Returns the name of the object
   */
  public String getName() {
    return (String)tag2name.get(new Integer(tag));
  }

  /**
   * Compares two ASN.1 types by tag
   */
  public boolean equals(Object o) {
    if (!(o instanceof ASN)) return false;
    ASN type = (ASN)o;
    return this.tag == type.tag;
  }

  /**
   * Returns hashcode for this object
   */
  public int hashCode() {
    return tag;
  }
}



