// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

/*
 * This is the standard class for dealing with ASN1 stuff
 *
 */

package cryptiX.asn1;

import java.util.*;
import java.io.*;

import cryptiX.asn1.*;
import cryptiX.utils.*;
import cryptiX.crypto.Md5Hash;

public class ASN1 {

    final public static int DER = 1;        // global value for format DER
    final public static int PEM = 2;        // global value for format PEM

    // Registry of OID <-> Name correspondance,
    static Hashtable oid2Name = new Hashtable(ASN.INITIAL_HASH_SIZE);

    // only one DER coder for all instances of this class
    private static BerCoder derCoder;

    // ASN1Object of this class
    private ASN1Object asn1Obj = null;

    // ASN1Object in original DER array
    private byte[] asn1Array = null;

    // file format of this ASN.1 object
    private int saveFormat = DER;           // default save format is DER

    static {
        // Infrastructure classes, eager loading
        Loader.loadClass("cryptiX.asn1.ASN");
        Loader.loadClass("cryptiX.asn1.ASN1Object");
        Loader.loadClass("cryptiX.asn1.BASIC");
        Loader.loadClass("cryptiX.asn1.SEQUENCE");
        Loader.loadClass("cryptiX.asn1.CONTEXT");
        Loader.loadClass("cryptiX.asn1.CodingException");

        // create one DER coder
        derCoder = new BerCoder();
    };

    /**
     * default constructor
     * To ensure, that classes are loaded in static {}.
     */
    public ASN1() {
    }

    /**
     * creates the ASN1Object from an other ASN1Object.
     */
    public ASN1(ASN1Object o) {
        asn1Obj = o;

        try {
            asn1Array = derCoder.encode(asn1Obj);       // try to DER encode
        } catch (CodingException ex) {
            System.out.println("DER encoding exception: "+ex.getMessage());
            asn1Array = null;
        }
    }

    /**
     * creates the ASN1Object from a file.
     * The format of the file could be PEM or DER.
     */
    public ASN1(String fileName) throws IOException, CodingException {

        fromByteArray(readKeyFile(fileName));
    }

    /**
     * creates the ASN1Object from a byte array.
     * The format of the array could be PEM or DER.
     */
    public ASN1(byte[] arr) throws CodingException {
        fromByteArray(arr);
    }

    /**
     * creates the ASN1Object from a byte array
     * This method first tries to Base64 decode the array. If this
     * doesn't work the array is only DER decoded.
     */
    private void fromByteArray(byte[] arr) throws CodingException {

        try {
            Base64Decoder b64d = new Base64Decoder(arr);

            asn1Array = b64d.processArray();        // try to BASE64 decode
        } catch (Base64FormatException ex) {
            asn1Array = arr;                        // maybe only DER
        }

        try {
            asn1Obj = derCoder.decode(asn1Array);   // try to DER decode
        } catch (CodingException ex) {
            asn1Obj = null;
            asn1Array = null;
            throw ex;
        }
    }

    public byte[] readKeyFile(String fileName) throws IOException {

        File file = new File(fileName);
        byte[] buffer = new byte[new Long(file.length()).intValue()];
        FileInputStream is = new FileInputStream(file);
        DataInputStream dis = new DataInputStream(is);

        String line = null;
   		while ((line = dis.readLine()) != null  && !line.startsWith("-----BEGIN"))
            ;

        if (line != null) {                     // file is PEM format

            saveFormat = PEM;

            int index=0;
            while (true) {
                line = dis.readLine();
                if (line.startsWith("-----END"))
                    break;
                line.getBytes(0, line.length(), buffer, index);
                index += line.length();
            }

            dis.close() ;

            byte[] tmp = new byte[index];       // truncate buffer to bytes read
            System.arraycopy(buffer, 0, tmp, 0, index);

            return tmp;                      // return tmp with PEM
        }

   		is = new FileInputStream(file);         // no PEM file, read in buffer

        saveFormat = DER;

        is.read(buffer);
        is.close();

        return buffer;
    }

    /**
     * Returns the ASN1Object at index if asn1Obj is a SEQUENCE.
     *
     * @param index position in the sequence
     * @return the ASN1Object at this position
     */
    public ASN1Object getComponentAt(int index) throws CodingException {

        if (asn1Obj.isA(ASN.SEQUENCE)) {        // this works only for a sequence
            return (ASN1Object)asn1Obj.getComponent(index);
        }
        else
            throw new CodingException("ASN1: Format Error!");
    }

    /**
     * returns the number of components for this ASN1Object.
     * Only useful with a SEQUENCE.
     */
    public int components() {
        return asn1Obj.countComponents();
    }

    /**
     * returns the first object within the first sequence
     *
     * is used for checking the signature from the original data
     */
    public byte[] getFirstObject() throws CodingException {

        int index = 0;

        if (asn1Array[index++] != 0x30)         // works only for a sequence
            throw new CodingException("Format error!");

        while (asn1Array[index] != 0x30)             // search first sequence (= signed object)
            index++;

        int start = index++;

        int[] end = new int[1];
        int length = BerCoder.decodeASN1Length(asn1Array, index, end);

        byte[] first = new byte[length+end[0]-start];
                                                // and this should be the signed data
        System.arraycopy(asn1Array, start, first, 0, length+end[0]-start);

        return first;
    }

    /**
     * returns the ASN1Object in DER format
     *
     * @return the ASN1Object in DER format
     */
    public byte[] toByteArrayDER() {

        return asn1Array;
    }

    /**
     * returns the ASN1Object in PEM format
     *
     * @return the ASN1Object in PEM format
     */
    public byte[] toByteArrayPEM() {

        Base64Encoder b64e = new Base64Encoder(asn1Array);
        byte [] arr = b64e.processArray();              // try to BASE64 encode

        return arr;
    }

    /**
     * saves this ASN1Object to a DER file.
     *
     * @param fileName the name of the file
     */
    public void saveToFile(String fileName) throws IOException {

        File file = new File(fileName);
        FileOutputStream os = new FileOutputStream(file);

        os.write(asn1Array);
        os.close();
    }

    /**
     * saves this ASN1Object to a PEM file.
     *
     * @param fileName the name of the file
     * @param begin the first line of the file
     * @param end the last line of the file
     */
    public void saveToFile(String fileName, String begin, String end) throws IOException  {

        File file = new File(fileName);
        String line;
        int len;

        DataOutputStream dos = new DataOutputStream(new FileOutputStream(file));

        byte[] pem = toByteArrayPEM();

        dos.writeBytes(begin);
        dos.writeBytes("\n");


        dos.write(pem);

        dos.writeBytes("\n");
        dos.writeBytes(end);
        dos.writeBytes("\n");
        dos.close();
    }

    /**
     * sets the file format (ASN1.DER, ASN1.PEM)
     */
    public void setSaveFormat(int format) {
        saveFormat = format;
    }

   /**
     * returns the file format (ASN1.DER, ASN1.PEM)
     */
    public int saveFormat() {
        return saveFormat;
    }

    /**
     * converts the ASN1Object to String
     * result is similar asn1parse
     */
    public String toString() {
        pre = new String("");
        return print(asn1Obj);
    }

    // prefix (spaces) for printing a line with print
    // is used to indent lines for easier reading
    private static String pre;

    /**
     * recurive method for toString();
     */
    public static String print(ASN1Object o) {

        pre = pre.concat("  ");

        if (o == null) {
            pre = pre.substring(2);
            return "";
        }

        String str = o.toString();
        str = str.concat("\n");

        for (int i=0; i<o.countComponents(); i++) {
            str = str.concat(pre);
            str = str.concat(print((ASN1Object)o.getComponent(i)));
        }

        pre = pre.substring(2);
        return str;
    }

    /**
     * returns the source ASN1Object
     */
    public ASN1Object toASN1Object() {
        return asn1Obj;
    }

    /**
     * returns a fingerprint (MD5 Hash of the whole cert) from this ASN1 object.
     * If possible, the original array is used.
     */
    public byte[] fingerprint() {

        Md5Hash hash = new Md5Hash();

        hash.add(asn1Array);

        return hash.get();
    }

    /**
     * registers a OID <-> name pair
     */
    public static void registerOID(String OID, String Name) {
        oid2Name.put(OID, Name);
    }

    /**
     * returns the name of an object id
     *
     * @param oid the object ID (e.g. "2.5.4.3")
     */
    public static String Oid2Name(String oid) {
        String name;

        name = (String)oid2Name.get(oid);

        if (name == null)
            return oid;
        else
            return name;
    }

}

