//
// ASN1Object.java --
//
// Author: Pekka Nikander <pnr@tequila.nixu.fi>
//
// Copyright 1996 by Nixu Oy, Helsinki, Finland
//
// Created: Wed Mar 13 17:18:43 1996
// Last modified: Sun Mar 31 23:26:43 1996 pnr
//
// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.asn1;

import cryptiX.asn1.ASN;
import cryptiX.utils.Composite;

/**
 * This abstract class defines the <EM>external</EM> interface for
 * ASN.1 values, both primitive and constructed.
 *
 * The subclasses of this class are assumed to define the application
 * specific methods for accessing the data stored in the ASN.1 data
 * structure.
 * <p>
 * This class also implements a number of abstract factory methods
 * that create components of appropriate subclasses.  In this way,
 * this class acts as a concrete factory an Abstract Factory pattern.
 */
/*
 * Note that we cannot define the abstract factory since Java doesn't
 * allow static messages to be defined in an interface.  We could
 * implement the creation routines as a distinct object, but that
 * would unnecessarily create a new class and run time object.
 */

public abstract class ASN1Object implements Composite {

  /******************************************************************
   * Component interface, i.e. common operations to all
   * ASN.1 nodes
   *****************************************************************/

  /**
   * Performs any class specific initialization.
   * An abstract method implemented by subclasses.
   */
  protected abstract void init(ASN type);

  /**
   * The ASN.1 type of this ASN.1 object.
   */
  protected ASN asnType;

  /**
   * Returns the ASN.1 type of this ASN.1 object.
   */
  public ASN getAsnType() {
    return asnType;
  }

  /**
   * Tests if this object is an instance of given ASN.1 type.
   */
  public boolean isA(ASN type) {
    return asnType.equals(type);
  }

  /**
   * Set the value of this component as a native Java object.
   */
  public void setValue(Object value) {
    throw new IllegalArgumentException();
  }

  /**
   * Get the value of this component as a native Java object.
   * @returns null if the object is not representable as a Java object.
   */
  public Object getValue() {
    return null;
  }

  /**
   * Returns the object value, used by the coders.
   */
  protected Object getValueInternal() {
    return null;
  }

  /**
   * Gets the classes in which formats the values may be returned.
   * @returns an array of classes
   */
  public Class[] getTypes()        { return new Class[0]; }


  /******************************************************************
   * cryptiX.utils.Composite interface
   *****************************************************************/

  /**
   * Add a new component to the end of sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public void addComponent(Composite component) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Add a new component to a given location within the sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public void addComponent(Composite component, int index) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Removes a component from a sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public void removeComponent(Composite component) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Removes a component at the given location within a sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public void removeComponent(int index) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Returns the component at the given location within a sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public cryptiX.utils.Composite getComponent(int index) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Sets the component at the given location within a sequence.
   *
   * Unless overridden by subclass, throws NonCompositeException.
   */
  public void setComponent(int index, cryptiX.utils.Composite value) {
    throw new cryptiX.utils.NonCompositeException();
  }

  /**
   * Returns the number of component objects.  Zero if none.
   */
  public int countComponents() {
    return 0;
  }

  /**
   * Returns an Enumeration that iterates over the component objects.
   * Returns a NullEnumeration if this object is not a container.
   */
  public java.util.Enumeration getComponents() {
    return new cryptiX.utils.NullEnumeration();
  }

}
