//
// BASIC.java --
//
// Author: Pekka Nikander <pnr@tequila.nixu.fi>
//
// Copyright 1996 by Nixu Oy, Helsinki, Finland
//
// Created: Tue Mar 19 17:01:32 1996
// Last modified: Sun Mar 31 22:51:09 1996 pnr
//
// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.asn1;

import java.util.Dictionary;
import java.util.Hashtable;
import cryptiX.utils.Convert;
import cryptiX.rsa.BigInt;


/**
 * This class represents ASN.1 basic and object types,
 * i.e. BOOLEANs, INTEGERs, (BIT STRINGSs ), OCTET STRINGs,
 * (REALs), (ENUMERATED objects), (OBJECT IDENTIFIERs) and
 * (Object Descriptors).
 */

final class BASIC extends ASN1Object {
  /*
   * The current Java syntax makes it impossible to use
   * Java class names as literals.  The following constans
   * circument this.  Ugly, but makes the rest of this class
   * cleaner...
   */
  private final static Class _Boolean = new Boolean(true).getClass();
  private final static Class _Double  = new Double(0.0).getClass();
  private final static Class _Float   = new Float(0.0).getClass();
  private final static Class _Integer = new Integer(0).getClass();
  private final static Class _BigInt = new BigInt(0).getClass();
  private final static Class _Long    = new Long(0).getClass();

  private final static Class _String  = new String().getClass();
  private final static Class _Object  = new Object().getClass();

  /**
   * Registry of ASN.1 type <-> Java type correspondance,
   * in both directions.
   */
  static Dictionary java2asn = new Hashtable(ASN.INITIAL_HASH_SIZE);
  static Dictionary asn2java = new Hashtable(ASN.INITIAL_HASH_SIZE);

  /**
   * Registers an ASN.1 type <-> Java type correspondence
   * that can be handled by the BASIC class.
   */
  protected synchronized static void register(ASN type, Class cl) {
    java2asn.put(cl, type);
    asn2java.put(type, cl);
  }

  /**
   * Initialized the object factory method and the
   * known ASN basic types to this class.
   */
  static {
    Class _BASIC = new BASIC().getClass();
    ASN.register(ASN.BOOLEAN, _BASIC);      register(ASN.BOOLEAN, _Boolean);
    ASN.register(ASN.INTEGER, _BASIC);      register(ASN.INTEGER, _BigInt);
    ASN.register(ASN.OCTET_STRING,  _BASIC); register(ASN.OCTET_STRING,  _Object);
    ASN.register(ASN.NULL,    _BASIC);      register(ASN.NULL,    _Object);
    ASN.register(ASN.OID,     _BASIC);      register(ASN.OID,     _String);

    ASN.register(ASN.BIT_STRING, _BASIC);   register(ASN.BIT_STRING, _Object);//WP
    ASN.register(ASN.IA5STRING,  _BASIC);   register(ASN.IA5STRING,  _String);//WP
    ASN.register(ASN.T61STRING,  _BASIC);   register(ASN.T61STRING,  _String);//WP
    ASN.register(ASN.PRINTABLESTRING,  _BASIC);    register(ASN.PRINTABLESTRING,  _String);//WP
    ASN.register(ASN.UTCTIME,  _BASIC);     register(ASN.UTCTIME,  _String);//WP
  }

  /**
   * The value of this object.
   */
  protected Object value;

  /**
   * The basic Java type for objects.
   */
  protected Class basictype;

  /**
   * Creates a new BASIC object.  Called by the OBJECTFactory.
   */
  protected BASIC() {
  }

  /**
   * Initializes the newly created object.
   */
  protected void init(ASN type) {
    this.asnType = type;
    this.value = null;
    this.basictype = (Class)asn2java.get(type);
  }

  /**
   * Sets the object value.
   */
  public synchronized void setValue(Object value) {
    Object newvalue;

    newvalue = Convert.convert(value, basictype);
    if (newvalue == null) {
      throw new IllegalArgumentException("Cannot convert "
					 + value.getClass().getName()
					 + " to a "
					 + basictype.getName());
    }
    this.value = newvalue;
  }

  /**
   * Gets the object value
   */
  public synchronized Object getValue() {
    return value;
  }

  /**
   * Gets the object value, used by the coders
   */
  protected Object getValueInternal() {
    return value;
  }

  /**
   * Returns the types of Java objects this ASN.1 object
   * can be represented as.
   */
  public synchronized Class[] getTypes() {
    /* Everything can be represented as string */
    Class[] types;

    int n = 1;
    if (basictype != _String)
      n++;
    types = new Class[n];
    types[--n] = _String;
    if (n > 0) {
      types[--n] = basictype;
    }
    return types;
  }

  /****************************************************************
   * java.lang.Object interface
   ****************************************************************/

  /**
   * Returns a String that represents the value of this Object.
   */
  public String toString() {

    if (asnType.equals(ASN.OID)) {
        String name = ASN1.Oid2Name((String)value);
        if (name !=null)
            return asnType.getName() +" = "+"[" + basictype.getName() + "]" + " = " + name;
    }

    return asnType.getName() +" = "+"[" + basictype.getName() + "]" + " = " + value;
  }
}
