// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.rsa;

import java.io.*;
import java.util.Observable;
import cryptiX.asn1.*;
import cryptiX.utils.*;

/*
** Class for handling public keys
**
*/
public class PublicKey extends Observable {

    BigInt modulus;             // modulus
    BigInt publicExponent;      // public exponent
    String ID;                  // ID of this key


    protected PublicKey() {
        modulus = null;
        publicExponent = null;
        ID = "";
    }


    /**
     * creates a public key with given values
     *
     * @param UserID user id of the public key
     * @param exponent the public exponent
     * @modulus the modulus
     */
    public PublicKey(String UserID, BigInt exponent, BigInt modulus) {
        this.modulus = modulus;
        publicExponent = exponent;
        ID = UserID;
    }

    /**
     * uses the public key en/decrypt a message
     *
     * @param message the message as BigInt
     * @return the en/decrypted message
     */
    public BigInt crypt(BigInt message) {

        return RSAFunctions.mExp(message, publicExponent, modulus);
    }

    /**
     * returns the modulus
     */
    public BigInt modulus() {
        return new BigInt(modulus);
    }

    /**
     * returns the public exponent
     */
    public BigInt publicExponent() {
        return new BigInt(publicExponent);
    }

    /**
     * returns true if the public keys are equal
     */
    public boolean equals (Object obj) {

	    return  (obj instanceof PublicKey) && modulus.equals(((PublicKey)obj).modulus) &&
	            publicExponent.equals(((PublicKey)obj).publicExponent);
    }

    /**
     * creates a public key from a SubjectPublicKeyInfo object
     * by calling the algorithm specified by OID
     *
     * @param spkiObj SubjectPublicKeyInfo as ASN1Object
     * @return a public key object
     */
    public static PublicKey valueOf(ASN1Object spkiObj) throws CodingException {

        ASN1 spki = new ASN1(spkiObj);                  // SubjectPublicKeyInfo

        String algorithm = Util.AlgIDAlgorithm(spki.getComponentAt(0));

        if (algorithm.equals(RSAFunctions.rsaEncryption)) {

            ASN1 spk = new ASN1((byte[])(spki.getComponentAt(1)).getValue());

            return new RSAPublicKey(spk.toASN1Object());    // RSA with SubjectPublicKey
        }
        else                                            // only RSA at this time
            throw new CodingException("SubjectPublicKeyAlgorithm "+algorithm+" not implemented!");
    }

    /**
     * returns the public key as a SubjectPublicKeyInfo structure (X.509)
     */
    public ASN1Object toSubjectPublicKeyInfo() {
        return null;                // implemented by derived classes
    }

    /**
     * returns the public key as a SubjectPublicKey structure (X.509)
     */
    public ASN1Object toSubjectPublicKey() {
        return null;                // implemented by derived classes
    }

    /**
     * checks a signature of a message
     * by calling the signature algorithm specified by OID
     *
     * @param message the message the signature applies to
     * @param signature the signature as defined by signature algorithm
     * @param signatureAlgorithm the algorithm used to generate this signature
     * @return true if signature is OK
     */
    public boolean checkSignature(byte[] message, byte[] signature, String signatureAlgorithm)
        throws CodingException {

        if (signatureAlgorithm.equals(RSAFunctions.md2WithRSAEncryption) ||
            signatureAlgorithm.equals(RSAFunctions.md4WithRSAEncryption) ||
            signatureAlgorithm.equals(RSAFunctions.md5WithRSAEncryption) ||
//            signatureAlgorithm.equals(RSAFunctions.md5) ||    // to test the SSLeay cert
            signatureAlgorithm.equals(RSAFunctions.sha1WithRSAEncryption)) {

            return RSAFunctions.checkSignature(message, signature, this);
        }
        else                                            // only RSA at this time
            throw new CodingException("Signature Algorithm "+signatureAlgorithm+" not implemented!");
    }

    /**
     * returns the ID of the public key
     */
    public String ID() {
        return ID;
    }

    /**
     * sets the ID of the public key
     */
    public void setID(String newID) {
        ID = newID;
    }

    /**
     * String representation of the public key
     */
    public String toString() {
        String key = new String("modulus:          "+modulus.toString(16)+"\n");
        return key.concat("public exponent:        "+publicExponent.toString(16)+"\n");
    }
}
