//
// Factory.class --
//
// Author: Pekka Nikander <pnr@tequila.nixu.fi>
//
// Copyright 1996 by Nixu Oy, Helsinki, Finland
//
// Created: Fri Mar 22 08:33:23 1996
// Last modified: Thu Apr 11 08:57:20 1996 pnr
//

package cryptiX.utils;

import  java.util.Dictionary;
import  java.util.Hashtable;

/**
 * The Factory provides a means to register and instantiate
 * concrete classes that implement a certain abstract class or
 * interface. The purpose of the factory is to facilitate runtime
 * binding of implementation classes.  The runtime system can
 * dynamically, or example from applet parameters, what classes to
 * load and instantiate.
 * <p>
 * In order to use the Factory, all instances of the implemented
 * classes shall be made indirectly, via the factory.  The concreate
 * instantiable classes must register themselves with the factory.
 * The runtime environment must load one or more instantiable
 * classes, since otherwise they may not be available...
 * <p>
 * The Factory class is an abstract class, to be subclassed
 * as the actual "Thing" factory.  There may be one factory
 * per package, or one factory per each dynamic interface.
 * In theory, it would be possible to construct a single
 * system wide factory class, but the Java runtime name
 * space checking prevents this.  More specificly, Class.newInstance()
 * does not create instances for classes that are protected
 * and in another package.  Therefore there must be at least
 * one factory per package.
 * <p>
 * For an example of a concrete factory, see FactorySimple.java.
 * For more discussion about Design Patterns, see [Gamma et al,
 * <em>Design Patterns</em>]
 */

public abstract class Factory {
  /**
   * Restrict instantiation to the class itself and subclasses.
   */
  protected Factory() {
  }

  /****************************************************************
   * Common internal routines.
   ****************************************************************/

  /**
   * Repository of known interfaces.  <Class,Hashtable>
   * pairs, where the Hashtable contains <Object,Class>
   * pairs for the known classes implementing the interface Class.
   */
  static Dictionary interfaces = new Hashtable();

  /**
   * Repository of Singleton instances.  Some concrete
   * classes desire that only one instance of them is ever
   * created, and that instance is always returned when asked.
   * This Dictionary holds <Class,Object> pairs for such
   * classes.
   */
  static Dictionary singletons = new Hashtable();

  /**
   * Creates a new instance of the given class.
   * This method must be implemented by a subclass
   * in order to behave with Java package protection.
   * <p>
   * This is the only non-static function to be implemented by
   * the subclasses.  Typically, the factory classes
   * will implement their own, class specific static routines
   * for instance creation.  In fact, by doing this clever
   * protection we have enforced this....
   * <p>
   * @param implementation the implementation class to be instantiated
   * @exception InstantiationException if the implementation class
   *		was not available, or if it happens to be an abstract
   *		class or an interface.
   * @exception IllegalAccessException if an attempt was made to
   *		instantiate a of a foreign package not supporting
   *		foreign instantiation.
   */
  protected abstract Object create(Class implementation)
    throws InstantiationException, IllegalAccessException;

  /**
   * Create an instance using the given interface, type and default.
   * @param interface The abstract superclass or interface
   * @param type An object denoting indirectly which concrete
   *             class should be used when instantiating
   * @param default Default type to be used if type == null
   * @param singleton Boolean denoting weather a shared instance
   *                  should be used.
   * @returns the newly created, uninitialized instance.
   * @exception InstantiationException if new instance couldn't be created.
   */
  protected synchronized Object
    create(Class iface, Object type, boolean singleton)
      throws InstantiationException {

    Dictionary classes;		// Dictionary of implementing classes
    Class implementation;	// Concrete class to instantiate
    Object instance;		// Instance to be created

    // Look up the implementation class colletion
    classes = (Dictionary)interfaces.get(iface);
    if (classes == null)
      throw new InstantiationException("No implementations known for class "
				       + iface.getName() + ".");

    // Look up the actual implementation class
    implementation = (Class)classes.get(type);
    if (implementation == null)
      throw new InstantiationException("No known implementation for " + type);

    // Look up the instance, if singleton
    if (singleton) {
      instance = singletons.get(implementation);
      if (instance != null)
	return instance;
    }

    // Create the desired object instance
    try {
      instance = create(implementation);
    } catch (IllegalAccessException e) {
      throw new InstantiationException("Error in creating instance for " + type
				       + " using class " + implementation
				       + ": " + e.getMessage());
    }

    // Store the instance, if singleton
    if (singleton)
      singletons.put(implementation, instance);

    return instance;
  }

  /**
   * Register a class as one providing implementations for
   * a given interface and type.
   * @param interface The desired interface class
   * @param type An Object used to denote implementation, typically a String
   * @param implementation The implementation class being registered
   */
  protected synchronized void register(Class iface,
				       Object type, Class implementation) {
    Dictionary classes;

    // Look up the implementation class collection, or create one
    classes = (Dictionary)interfaces.get(iface);
    if (classes == null) {
      classes = new Hashtable();
      interfaces.put(iface, classes);
    }

    // Store the implementation
    classes.put(type, implementation);
  }
}


