// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.utils;

import cryptiX.asn1.*;

/**
 * This class represents a X.500 Name structure.
 */

public class Name {

    public static String commonNameOID =                new String("2.5.4.3");
    public static String countryOID =                   new String("2.5.4.6");
    public static String localityOID =                  new String("2.5.4.7");
    public static String stateOrProvinceOID =           new String("2.5.4.8");
    public static String organizationOID =              new String("2.5.4.10");
    public static String organizationalUnitOID =        new String("2.5.4.11");
    public static String EmailAddressOID =              new String("1.2.840.113549.1.9.1");

    private String commonName = null;
    private String country = null;
    private String locality = null;
    private String stateOrProvince = null;
    private String organization = null;
    private String organizationalUnit = null;

    static {
        ASN1.registerOID("2.5.4.3", "Common Name");
        ASN1.registerOID("2.5.4.6", "Country");
        ASN1.registerOID("2.5.4.7", "Locality");
        ASN1.registerOID("2.5.4.8", "State Or Province");
        ASN1.registerOID("2.5.4.10","Organization");
        ASN1.registerOID("2.5.4.11","Organizational Unit");
    }

    /**
     * default constructor
     */
    public Name() {
    }

    /**
     * constructor
     */
    public Name(String c, String sp, String l, String o, String ou, String cn) {
        if (c.length() > 0)  country = c;
        if (sp.length() > 0) stateOrProvince = sp;
        if (l.length() > 0)  locality = l;
        if (o.length() > 0)  organization = o;
        if (ou.length() > 0) organizationalUnit = ou;
        if (cn.length() > 0) commonName = cn;
    }

    /**
     * constructor for ASN1Object: SEQUENCE OF RelativeDistinguishedName
     */
    public Name(ASN1Object name) throws CodingException {

        ASN1Object set;

        if (!name.isA(ASN.SEQUENCE))            // this works only for a sequence
            return; // @todo throw exception

        for (int i=0; i<name.countComponents(); i++) {
            set = (ASN1Object)name.getComponent(i);
            parseOID((ASN1Object)set.getComponent(0));
        }
    }

    /**
     * parses an AttributeValueAssertion and sets the value
     *
     * @param ava the AttributeValueAssertion ASN1Object (sequence)
     */
    private void parseOID(ASN1Object ava) throws CodingException {

        ASN1 AVA = new ASN1(ava);

        if (AVA.getComponentAt(0).getValue().equals(commonNameOID))
            commonName = (String)AVA.getComponentAt(1).getValue();
        else if (AVA.getComponentAt(0).getValue().equals(countryOID))
            country = (String)AVA.getComponentAt(1).getValue();
        else if (AVA.getComponentAt(0).getValue().equals(localityOID))
            locality = (String)AVA.getComponentAt(1).getValue();
        else if (AVA.getComponentAt(0).getValue().equals(stateOrProvinceOID))
            stateOrProvince = (String)AVA.getComponentAt(1).getValue();
        else if (AVA.getComponentAt(0).getValue().equals(organizationOID))
            organization = (String)AVA.getComponentAt(1).getValue();
        else if (AVA.getComponentAt(0).getValue().equals(organizationalUnitOID))
            organizationalUnit = (String)AVA.getComponentAt(1).getValue();
    }

    private ASN1Object generateSet(String OID, String value) {
        ASN1Object set = null;
        ASN1Object seq = null;

        try {
            set = ASN.create(ASN.SET);
            seq = ASN.create(ASN.SEQUENCE);

            seq.addComponent(ASN.create(ASN.OID, OID));
            seq.addComponent(ASN.create(ASN.PRINTABLESTRING, value));

            set.addComponent(seq);
        } catch (InstantiationException ex) {
            System.out.println(ex.getMessage());
        }

        return set;
    }


    public ASN1Object toASN1Object() {
        ASN1Object name = null;

        try {
            name = ASN.create(ASN.SEQUENCE);
        } catch (InstantiationException ex) {
            System.out.println(ex.getMessage());
        }

        if (country != null)
            name.addComponent(generateSet("2.5.4.6", country));
        if (stateOrProvince != null)
            name.addComponent(generateSet("2.5.4.8", stateOrProvince));
        if (locality != null)
            name.addComponent(generateSet("2.5.4.7", locality));
        if (organization != null)
            name.addComponent(generateSet("2.5.4.10", organization));
        if (organizationalUnit != null)
            name.addComponent(generateSet("2.5.4.11", organizationalUnit));
        if (commonName != null)
            name.addComponent(generateSet("2.5.4.3", commonName));

        return name;
    }

    public String commonName() {
        return commonName;
    }

    public String country() {
        return country;
    }

    public String locality() {
        return locality;
    }

    public String stateOrProvince() {
        return stateOrProvince;
    }

    public String organization() {
        return organization;
    }

    public String organizationalUnit() {
        return organizationalUnit;
    }

    public void setCommonName(String cn) {
        if (cn.length() > 0) commonName = cn;
    }

    public void setCountry(String c) {
        if (c.length() > 0) country = c;
    }

    public void setLcality(String l) {
        if (l.length() > 0) locality = l;
    }

    public void setStateOrProvince(String sp) {
        if (sp.length() > 0) stateOrProvince = sp;
    }

    public void setOrganization(String o) {
        if (o.length() > 0) organization = o;
    }

    public void setOrganizationalUnit(String ou) {
        if (ou.length() > 0) organizationalUnit = ou;
    }

    public String toString() {

        String str = new String();

        if (country != null)
            str = str.concat("C="+country+", ");
        if (stateOrProvince != null)
            str = str.concat("SP="+stateOrProvince+", ");
        if (locality != null)
            str = str.concat("L="+locality+", ");
        if (organization != null)
            str = str.concat("O="+organization+", ");
        if (organizationalUnit != null)
            str = str.concat("OU="+organizationalUnit+", ");
        if (commonName != null)
            str = str.concat("CN="+commonName+", ");

        return str.substring(1);                    // cut last ,
    }
}

